"break" @keyword

"case" @keyword

"const" @keyword

"continue" @keyword

"default" @keyword

"do" @keyword

"else" @keyword

"enum" @keyword

"extern" @keyword

"for" @keyword

"if" @keyword

"inline" @keyword

"return" @keyword

"sizeof" @keyword

"static" @keyword

"struct" @keyword

"switch" @keyword

"typedef" @keyword

"union" @keyword

"volatile" @keyword

"while" @keyword

"#define" @keyword

"#elif" @keyword

"#else" @keyword

"#endif" @keyword

"#if" @keyword

"#ifdef" @keyword

"#ifndef" @keyword

"#include" @keyword

(preproc_directive) @keyword

"--" @operator

"-" @operator

"-=" @operator

"->" @operator

"=" @operator

"!=" @operator

"*" @operator

"&" @operator

"&&" @operator

"+" @operator

"++" @operator

"+=" @operator

"<" @operator

"==" @operator

">" @operator

"||" @operator

"." @delimiter

";" @delimiter

(string_literal) @string

(system_lib_string) @string

(null) @constant

(number_literal) @number

(char_literal) @number

(call_expression
  function: (identifier) @function)

(call_expression
  function: (field_expression
    field: (field_identifier) @function))

(function_declarator
  declarator: (identifier) @function)

(preproc_function_def
  name: (identifier) @function.special)

(field_identifier) @property

(statement_identifier) @label

(type_identifier) @type

(primitive_type) @type

(sized_type_specifier) @type

((identifier) @constant
  (#match? @constant "^[A-Z][A-Z\\d_]*$"))

(identifier) @variable

(comment) @comment

; inherits: c

[
  "in"
  "out"
  "inout"
  "uniform"
  "shared"
  "layout"
  "attribute"
  "varying"
  "buffer"
  "coherent"
  "readonly"
  "writeonly"
  "precision"
  "highp"
  "mediump"
  "lowp"
  "centroid"
  "sample"
  "patch"
  "smooth"
  "flat"
  "noperspective"
  "invariant"
  "precise"
] @type.qualifier

"subroutine" @keyword.function

(extension_storage_class) @storageclass

((identifier) @variable.builtin
  (#match? @variable.builtin "^gl_"))
